
#include "hal_nrf.h"


#include "hal_uart.h"
#include "hal_clk.h"

#include <stdio.h>

#include "nordic_common.h"

// Global variables
uint8_t payload[32];

uint8_t packetId;

bool got;			// flag that indicates whether a packet was received.
bool tx_err;	
uint8_t pid;	// packet id


// Cusomization of low level stdio function. Used by for example printf().
#ifdef __ICC8051__
int putchar(int c)
#else /*presume C51 or other accepting compilator*/
char putchar(char c)
#endif
{
  hal_uart_putchar(c);
  return c;
}

// Cusomization of low level stdio function. Used by for example gets().
#ifdef __ICC8051__
//	#define getchar(x) ((int) hal_uart_getchar())
#else
//	#define getchar(x) ((char) hal_uart_getchar())
#endif


#ifdef __ICC8051__
//int getchar(void)   { return hal_uart_getchar();}
#else /*presume C51 or other accepting compilator*/
//char getchar(void) {  return hal_uart_getchar();}
#endif

// Repeated putchar to print a string
void putstring(char *s)
{
  while(*s != 0)
    putchar(*s++);
}

void uart_begin(void) {
	EA = 1;
	P0DIR = (P0DIR & ~0x08) | 0x10; // bxxx1-0xxx
	hal_uart_init(UART_BAUD_38K4);     //< must be 38.4 kbaud -- why? max for 
  while(hal_clk_get_16m_source() != HAL_CLK_XOSC16M);
}

/**
 flush all data in the buffer and close the connection
*/
void uart_end(void) {
	while(!hal_uart_tx_complete());
	REN0 = 0; // disable serial reception
	//S0CON &= ~0x;
}

/**
	wait for uart buffer to be empty
*/
void wait_uart(void) {
		while(!hal_uart_tx_complete());
}


void main()
{
	got = false;
	
	uart_begin();
  #ifdef MCU_NRF24LU1P
  
  // Enable the radio clock
  RFCKEN = 1;

  // Enable RF interrupt
  RF = 1;  
  // Enable global interrupt
  EA = 1;


  // Configure radio as primary receiver (PRX) 
  hal_nrf_set_operation_mode(HAL_NRF_PRX);

	hal_nrf_set_datarate(HAL_NRF_250KBPS);

  // Set payload width to 3 bytes
  hal_nrf_set_rx_payload_width(HAL_NRF_PIPE0, 32);


	
  // Set up dynamic payload on pipes
  hal_nrf_setup_dynamic_payload(0xFF);
  // Enable dynamic payload
  hal_nrf_enable_dynamic_payload(1);
  hal_nrf_enable_ack_payload(1);
  hal_nrf_enable_dynamic_ack(1);

  // Power up radio
  hal_nrf_set_power_mode(HAL_NRF_PWR_UP);

  // Enable receiver 
  CE_HIGH();
	printf("\n\n\n\nReady to go!!\n");
	wait_uart();
  while(1)
  { 
		if (got) {
			printf("got packet!\n");
			got = false;
		}
  } 
} 

// Radio interrupt
void rf_irq() interrupt INTERRUPT_RFIRQ
{
  uint8_t irq_flags;
   
  // Read and clear IRQ flags from radio
  irq_flags = hal_nrf_get_clear_irq_flags(); 
  
  // If data received
  if(irq_flags & (1<<HAL_NRF_RX_DR))
  {		
    // Read payload
    while(!hal_nrf_rx_fifo_empty())
    {
      hal_nrf_read_rx_payload(payload);
    }
    // Bounce back data with ACK
		
		// CHANGE some data, return as ack payload. 
		// this is NOT executed, for some reason, when PTX sends packet, sleeps, sends packet, etc.
		// it IS executed when PTX sends packet repeatedly without going into sleep mode.
		
		payload[0]=pid++; payload[1]=2; payload[2]=3;payload[3]=4;
    hal_nrf_write_ack_payload(0,payload,32);
		got = true;
  }
}